<?php

declare(strict_types=1);

namespace Drupal\tailwindcss_utility\form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\FocusFirstCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Database\Query\Merge;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\tailwindcss_utility\RuleStorage;
use Symfony\Component\DependencyInjection\ContainerInterface;

final class StylesForm extends FormBase {

  private const WRAPPER_ID = 'tailwind-css-rules-wrapper';

  /**
   * Tailwind rule storage service.
   *
   * @see https://www.drupal.org/project/drupal/issues/3110266
   * @phpstan-ignore-next-line
   */
  protected RuleStorage $ruleStorage;

  public function __construct(RuleStorage $rule_storage) {
    $this->ruleStorage = $rule_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): StylesForm {
    return new static(
      $container->get('tailwindcss_utility.rule_storage'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'tailwindcss_utility_styles_form';
  }

  /**
   * {@inheridoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $form['class'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Class'),
      '#description' => $this->t('Enter class name, if the class exists its rules will be editable.'),
      '#autocomplete_route_name' => 'tailwindcss_utility.class_autocomplete',
      '#autocomplete_route_parameters' => [
        'include_core' => TRUE,
      ],
      '#ajax' => [
        'callback' => [$this, 'ajax'],
      ],
      '#required' => TRUE,
    ];

    $form['wrapper'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => self::WRAPPER_ID,
      ],
    ];

    // @todo Support multiple rules.
    $form['wrapper']['rule'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Rule'),
      '#rows' => 3,
      '#required' => TRUE,
    ];
    $form['wrapper']['atrule'] = [
      '#type' => 'textarea',
      '#title' => $this->t('At - rule'),
      '#rows' => 3,
    ];

    $class = $form_state->getValue('class');
    $rule = [];
    if (!\is_null($class) && \strlen($class)) {
      $rule = $this->ruleStorage->getRule($class);
    }
    if (\count($rule) === 0) {
      $rule = [0 => ['rule' => '']];
    }
    $form_state->set('rule', $rule);

    $form['wrapper']['actions'] = ['#type' => 'actions'];
    if ($rule) {
      $form['wrapper']['actions']['save'] = [
        '#type' => 'submit',
        '#value' => $this->t('Save'),
      ];
      $form['wrapper']['actions']['delete'] = [
        '#type' => 'submit',
        '#value' => $this->t('Delete / Revert to default'),
      ];
    }
    else {
      $form['wrapper']['actions']['save'] = [
        '#type' => 'submit',
        '#value' => $this->t('Add rule'),
      ];
    }

    return $form;
  }

  /**
   * AJAX callback.
   */
  public function ajax(array $form, FormStateInterface $form_state): AjaxResponse {
    $rule = $form_state->get('rule');
    foreach (['rule', 'atrule'] as $key) {
      $value = \array_key_exists($key, $rule[0]) ? $rule[0][$key] : '';
      $form['wrapper'][$key]['#value'] = $value;
    }

    // Without custom handling below, the class element always got focus
    // after changing its value.
    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand('#' . self::WRAPPER_ID, $form['wrapper']));
    $response->addCommand(new FocusFirstCommand('#' . self::WRAPPER_ID));

    return $response;
  }

  /**
   * {@inheridoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $op = $form_state->getTriggeringElement()['#parents'][0];
    $class = $form_state->getValue('class');
    if ($op === 'save') {
      // @todo: support adding multiple rules
      $rule = [
        0 => [
          'rule' => $form_state->getValue('rule'),
          'atrule' => $form_state->getValue('atrule'),
        ],
      ];

      $result = $this->ruleStorage->addRule($class, $rule);
      if ($result === Merge::STATUS_INSERT) {
        $this->messenger()->addStatus($this->t('New rule for "@class" class added.', [
          '@class' => $class,
        ]));
      }
      elseif ($result === Merge::STATUS_UPDATE) {
        $this->messenger()->addStatus($this->t('Rule for "@class" class updated.', [
          '@class' => $class,
        ]));
      }
    }

    elseif ($op === 'delete') {
      $result = $this->ruleStorage->deleteRule($class);
      if ($result) {
        $this->messenger()->addStatus($this->t('Rule for "@class" class deleted.', [
          '@class' => $class,
        ]));
      }
    }

  }

}
