<?php

declare(strict_types=1);

namespace Drupal\tailwindcss_utility;

use Drupal\Core\Asset\AttachedAssetsInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\file\FileRepositoryInterface;
use Drupal\file\FileStorageInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;
use Psr\Log\LoggerInterface;

final class TailwindJsHandler {

  private const TAILWIND_JS_FILE_URI = 'public://tailwindcss_utility/tailwind.js';
  // @todo Make plugins configurable?
  private const TAILWIND_CDN_URL = 'https://cdn.tailwindcss.com?plugins=forms,typography,aspect-ratio,line-clamp';

  private FileRepositoryInterface $fileRepository;
  private FileSystemInterface $fileSystem;
  private ClientInterface $httpClient;
  private FileStorageInterface $fileStorage;
  private LoggerInterface $logger;

  public function __construct(
    FileRepositoryInterface $file_repository,
    FileSystemInterface $file_system,
    ClientInterface $http_client,
    EntityTypeManagerInterface $entity_type_manager,
    LoggerInterface $logger
  ) {
    $this->fileRepository = $file_repository;
    $this->fileSystem = $file_system;
    $this->httpClient = $http_client;
    $this->fileStorage = $entity_type_manager->getStorage('file');
    $this->logger = $logger;
  }

  public function deleteTailwindJs(): void {
    $files = $this->fileStorage->loadByProperties(['uri' => self::TAILWIND_JS_FILE_URI]);
    if (\count($files) > 0) {
      $file = \reset($files);
      $file->delete();
    }
    if (\file_exists(self::TAILWIND_JS_FILE_URI)) {
      $this->fileSystem->unlink(self::TAILWIND_JS_FILE_URI);
    }
  }

  public function fetchTailwindJs(): bool {
    $files = $this->fileStorage->loadByProperties(['uri' => self::TAILWIND_JS_FILE_URI]);
    if (\count($files) > 0) {
      return TRUE;
    }

    $directory = \substr(self::TAILWIND_JS_FILE_URI, 0, \strrpos(self::TAILWIND_JS_FILE_URI, '/'));
    if (!$this->fileSystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS)) {
      $this->logger->error('Unable to prepare direstory %directory for Tailwind JS file.', [
        '%directory' => $directory,
      ]);
      return FALSE;
    }
    try {
      $response = $this->httpClient->request('GET', self::TAILWIND_CDN_URL);
    }
    catch (GuzzleException $e) {
      $this->logger->error('An error occurred while fetching Tailwind JS from CDN: %error', [
        '%error' => $e->getMessage,
      ]);
      return FALSE;
    }
    if ($response->getStatusCode() !== 200) {
      $this->logger->error('An error occurred while fetching Tailwind JS from CDN (response code %code).', [
        '%code' => $response->getStatusCode(),
      ]);
      return FALSE;
    }

    $js_contents = $response->getBody()->getContents();
    $file = $this->fileRepository->writeData($js_contents, self::TAILWIND_JS_FILE_URI, FileSystemInterface::EXISTS_REPLACE);
    $file->setPermanent();
    $file->save();

    return TRUE;
  }

  public function alterScripts(array &$javascript, AttachedAssetsInterface $assets): void {
    foreach ($javascript as $key => $item) {
      if ($item['type'] !== 'external') {
        continue;
      }
      if (\strpos($item['data'], 'cdn.tailwindcss.com') !== FALSE) {
        if (!$this->fetchTailwindJs()) {
          break;
        }
        $javascript[$key]['type'] = 'file';
        $javascript[$key]['data'] = self::TAILWIND_JS_FILE_URI;
      }
    }
  }

}
